# coding=utf-8
import os
import shutil
import tempfile
from unittest import TestCase
import mock
from mock import Mock

from earthdiagnostics.obsreconmanager import ObsReconManager
from earthdiagnostics.modelingrealm import ModelingRealms


class TestObsReconManager(TestCase):

    def setUp(self):
        self.config = Mock()
        self.config.data_convention = 'specs'
        self.config.data_type = 'exp'
        self.config.experiment.expid = 'expid'
        self.config.experiment.model = 'model'
        self.config.experiment.experiment_name = 'expname'
        self.config.experiment.institute = 'institute'
        self.config.experiment.member_count_start = 0
        self.config.experiment.chunk_size = 12
        self.config.experiment.num_chunks = 1
        self.config.experiment.calendar = 'standard'
        self.config.experiment.atmos_timestep = 3
        self.config.experiment.ocean_timestep = 6

        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'mon'
        frequency.folder_name.return_value = 'monthly_mean'

        self.config.frequency = frequency

        self.config.cmor.initialization_number = 1
        self.config.cmor.version = ''
        self.config.cmor.default_ocean_grid = 'ocean_grid'
        self.config.cmor.default_atmos_grid = 'atmos_grid'
        self.config.cmor.activity = 'activity'
        self.config.cmor.force = False
        self.config.cmor.force_untar = False

        self.tmp_dir = tempfile.mkdtemp()
        self._make_data_folder()
        self.config.data_dir = self.tmp_dir

    def _make_data_folder(self):
        os.makedirs(os.path.join(self.tmp_dir, self.config.data_type, self.config.experiment.institute,
                                 self.config.experiment.model, self.config.experiment.expid))

    def tearDown(self):
        if os.path.exists(self.tmp_dir):
            shutil.rmtree(self.tmp_dir)

    def test_not_data_dir(self):
        shutil.rmtree(self.tmp_dir)
        with self.assertRaises(Exception):
            ObsReconManager(self.config)

    def test_bad_num_chunks(self):
        self.config.experiment.chunk_size = 3
        self.config.data_type = 'obs'
        self._make_data_folder()
        with self.assertRaises(Exception):
            ObsReconManager(self.config)

    def test_get_file_path(self):
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'
        frequency.folder_name.return_value = 'folder_name'

        manager = ObsReconManager(self.config)
        self.assertEqual(manager.get_file_path('19900101', ModelingRealms.ocean, 'var', frequency, None),
                         os.path.join(self.tmp_dir, 'exp/institute/model/folder_name/var/var_19900101.nc'))

    def test_get_file_path_default_freq(self):
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon

        manager = ObsReconManager(self.config)
        self.assertEqual(manager.get_file_path('19900101', ModelingRealms.ocean, 'var', None, None),
                         os.path.join(self.tmp_dir, 'exp/institute/model/monthly_mean/var/var_19900101.nc'))

    def test_get_file_path_no_startdate(self):
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon

        manager = ObsReconManager(self.config)
        self.assertEqual(manager.get_file_path(None, ModelingRealms.ocean, 'var', None, None),
                         os.path.join(self.tmp_dir, 'exp/institute/model/monthly_mean/var/var.nc'))

    def test_get_file_path_recon_weekly(self):
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon

        self.config.data_type = 'recon'
        self.config.experiment.chunk_size = 1
        self._make_data_folder()

        manager = ObsReconManager(self.config)
        self.assertEqual(manager.get_file_path('19900101', ModelingRealms.ocean, 'var', None, None),
                         os.path.join(self.tmp_dir, 'recon/institute/model/monthly_mean/var/var_199001.nc'))

    @mock.patch('earthdiagnostics.obsreconmanager.ObsReconManager.get_file_path')
    def test_declare_chunk(self, mock_get_file_path):
        mock_get_file_path.return_value = '/path/to/file'
        cmor_manager = ObsReconManager(self.config)
        datafile = cmor_manager.declare_chunk(ModelingRealms.ocean, 'var', '20010101', 1, 1)
        self.assertEqual(datafile.remote_file, '/path/to/file')

        mock_frequency = Mock()
        datafile = cmor_manager.declare_chunk(ModelingRealms.ocean, 'var', '20010101', 1, 1,
                                              frequency=mock_frequency)
        self.assertEqual(datafile.remote_file, '/path/to/file')

        self.config.var_manager.get_variable.return_value = None
        datafile = cmor_manager.declare_chunk(ModelingRealms.ocean, 'var', '20010101', 1, 1)
        self.assertEqual(datafile.remote_file, '/path/to/file')

    @mock.patch('earthdiagnostics.obsreconmanager.ObsReconManager.get_file_path')
    def test_request_chunk(self, mock_get_file_path):
        mock_get_file_path.return_value = '/path/to/file'
        cmor_manager = ObsReconManager(self.config)
        datafile = cmor_manager.request_chunk(ModelingRealms.ocean, 'var', '20010101', 1, 1)
        self.assertEqual(datafile.remote_file, '/path/to/file')




